@@ -7,14 +7,15 @@ as GL_TRIANGLES, and trimesh.Path2D/Path3D as GL_LINES
 """
 
 import base64
-import collections
 import json
+from collections import OrderedDict, defaultdict, deque
 
 import numpy as np
 
 from .. import rendering, resources, transformations, util, visual
 from ..caching import hash_fast
 from ..constants import log, tol
+from ..typed import NDArray, Optional
 from ..util import unique_name
 from ..visual.gloss import specular_to_pbr
 
@@ -203,10 +204,6 @@ def export_glb(
         extension_webp=extension_webp,
     )
 
-    # allow custom postprocessing
-    if tree_postprocessor is not None:
-        tree_postprocessor(tree)
-
     # A bufferView is a slice of a file
     views = _build_views(buffer_items)
 
@@ -218,6 +215,10 @@ def export_glb(
         tree["buffers"] = [{"byteLength": len(buffer_data)}]
         tree["bufferViews"] = views
 
+    # allow custom postprocessing
+    if tree_postprocessor is not None:
+        tree_postprocessor(tree)
+
     # export the tree to JSON for the header
     content = util.jsonify(tree, separators=(",", ":"))
     # add spaces to content, so the start of the data
@@ -484,7 +485,7 @@ def _buffer_append(ordered, data):
 
     Parameters
     ----------
-    od : collections.OrderedDict
+    od : OrderedDict
       Keyed like { hash : data }
     data : bytes
       To be stored
@@ -505,19 +506,19 @@ def _buffer_append(ordered, data):
     return len(ordered) - 1
 
 
-def _data_append(acc, buff, blob, data):
+def _data_append(acc: OrderedDict, buff: OrderedDict, blob: dict, data: NDArray):
     """
     Append a new accessor to an OrderedDict.
 
     Parameters
     ------------
-    acc : collections.OrderedDict
+    acc
       Collection of accessors, will be mutated in-place
-    buff : collections.OrderedDict
+    buff
       Collection of buffer bytes, will be mutated in-place
-    blob : dict
+    blob
       Candidate accessor
-    data : numpy.array
+    data
       Data to fill in details to blob
 
     Returns
@@ -562,6 +563,11 @@ def _data_append(acc, buff, blob, data):
     # see if we're an array, matrix, etc
     kind = blob["type"]
 
+    if tol.strict:
+        # in unit tests make sure everything we're trying to export
+        # is finite, which also checks for accidental NaN values
+        assert np.isfinite(data).all()
+
     if kind == "SCALAR":
         # is probably (n, 1)
         blob["count"] = int(np.prod(data.shape))
@@ -665,7 +671,7 @@ def _create_gltf_structure(
         "scene": 0,
         "scenes": [{"nodes": [0]}],
         "asset": {"version": "2.0", "generator": "https://github.com/mikedh/trimesh"},
-        "accessors": collections.OrderedDict(),
+        "accessors": OrderedDict(),
         "meshes": [],
         "images": [],
         "textures": [],
@@ -689,7 +695,7 @@ def _create_gltf_structure(
     # store materials as {hash : index} to avoid duplicates
     mat_hashes = {}
     # store data from geometries
-    buffer_items = collections.OrderedDict()
+    buffer_items = OrderedDict()
 
     # map the name of each mesh to the index in tree['meshes']
     mesh_index = {}
@@ -769,7 +775,7 @@ def _append_mesh(
     name,
     tree,
     buffer_items,
-    include_normals: bool,
+    include_normals: Optional[bool],
     unitize_normals: bool,
     mat_hashes: dict,
     extension_webp: bool,
@@ -922,10 +928,7 @@ def _append_mesh(
     ):
         # store vertex normals if requested
         if unitize_normals:
-            normals = mesh.vertex_normals.copy()
-            norms = np.linalg.norm(normals, axis=1)
-            if not util.allclose(norms, 1.0, atol=1e-4):
-                normals /= norms.reshape((-1, 1))
+            normals = util.unitize(mesh.vertex_normals)
         else:
             # we don't have to copy them since
             # they aren't being altered
@@ -978,7 +981,7 @@ def _build_views(buffer_items):
 
     Parameters
     --------------
-    buffer_items : collections.OrderedDict
+    buffer_items : OrderedDict
       Buffers to build views for
 
     Returns
@@ -1078,11 +1081,7 @@ def _byte_pad(data, bound=4):
         result = b"".join([data, pad])
         # we should always divide evenly
         if tol.strict and (len(result) % bound) != 0:
-            raise ValueError(
-                "byte_pad failed! ori:{} res:{} pad:{} req:{}".format(
-                    len(data), len(result), count, bound
-                )
-            )
+            raise ValueError("byte_pad failed!")
         return result
     return data
 
@@ -1461,9 +1460,9 @@ def _read_buffers(
         else:
             materials = _parse_materials(header, views=views, resolver=resolver)
 
-    mesh_prim = collections.defaultdict(list)
+    mesh_prim = defaultdict(list)
     # load data from accessors into Trimesh objects
-    meshes = collections.OrderedDict()
+    meshes = OrderedDict()
 
     # keep track of how many times each name has been attempted to
     # be inserted to avoid a potentially slow search through our
@@ -1661,9 +1660,9 @@ def _read_buffers(
     names[base_frame] = base_frame
 
     # visited, kwargs for scene.graph.update
-    graph = collections.deque()
+    graph = deque()
     # unvisited, pairs of node indexes
-    queue = collections.deque()
+    queue = deque()
 
     if "scene" in header:
         # specify the index of scenes if specified
@@ -2054,9 +2053,9 @@ def get_schema():
     from ..schemas import resolve
 
     # get a blob of a zip file including the GLTF 2.0 schema
-    blob = resources.get("schema/gltf2.schema.zip", decode=False)
+    stream = resources.get_stream("schema/gltf2.schema.zip")
     # get the zip file as a dict keyed by file name
-    archive = util.decompress(util.wrap_as_stream(blob), "zip")
+    archive = util.decompress(stream, "zip")
     # get a resolver object for accessing the schema
     resolver = ZipResolver(archive)
     # get a loaded dict from the base file
